// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  crystal growth group                                                  ==
// ==                                                                        ==
// ==  Stiftung caesar                                                       ==
// ==  Ludwig-Erhard-Allee 2                                                 ==
// ==  53175 Bonn                                                            ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  http://www.caesar.de/cg/AMDiS                                         ==
// ==                                                                        ==
// ============================================================================

/** \file GlobalDOFNumbering.h */

#ifndef AMDIS_GLOBALDOFNUMBERING_H
#define AMDIS_GLOBALDOFNUMBERING_H

// ===========================================================================
// ===== includes ============================================================
// ===========================================================================
#include <map>
#include <vector>
#include "Global.h"

namespace AMDiS {

  // =========================================================================
  // ===== forward declarations ==============================================
  // =========================================================================
  class MeshStructure;

  // =========================================================================
  // ===== class GlobalDOFNumbering ======================================
  // =========================================================================

  /** \ingroup Parallel
   * \brief 
   * Creates a global DOF numbering.
   */
  class GlobalDOFNumbering
  {
  public:
    /** \brief
     * Constructor
     */
    GlobalDOFNumbering(::std::vector<MeshStructure*> &meshCodes,
		       ::std::vector< ::std::vector<DegreeOfFreedom> > &dofCodes,
		       int dofsPerElement);

    /** \brief
     * Returns the local index of rank corresponding to global index.
     */
    int getLocalIndex(int rank, int globalIndex);

    /** \brief
     * Returns the global index corresponing to local index of rank.
     */
    int getGlobalIndex(int rank, int localIndex);

  protected:
    void handleRecursive(::std::vector<MeshStructure*> &meshCodes,
			 ::std::vector< ::std::vector<DegreeOfFreedom>::iterator> &it);

  protected:
    ::std::vector< ::std::map<DegreeOfFreedom, DegreeOfFreedom> > localToGlobal_;

    ::std::vector< ::std::map<DegreeOfFreedom, DegreeOfFreedom> > globalToLocal_;

    int dofsPerElement_;

    DegreeOfFreedom globalIndexCounter_;
  };

}

#endif
