#include "AMDiS.h"

using namespace AMDiS;
using namespace std;

class N : public AbstractFunction<double, WorldVector<double> >
{
public:
  MEMORY_MANAGED(N);

  const double& operator()(const WorldVector<double>& x) const
  {
    static double result = 1.0;
    return result;  
  };
};

// ===== function definitions // 
/** \brief
 * Dirichlet boundary function
 */
class G : public AbstractFunction<double, WorldVector<double> >
{
public:
  MEMORY_MANAGED(G);

  /** \brief
   * Implementation of AbstractFunction::operator().
   */
  const double& operator()(const WorldVector<double>& x) const
  {
    static double result;
    result = exp(-10.0*(x*x));
    return result;  
  };
};

/** \brief
 * RHS function
 */
class F : public AbstractFunction<double, WorldVector<double> >
{
public:
  MEMORY_MANAGED(F);

  F(int degree) : AbstractFunction<double, WorldVector<double> >(degree) {};

  /** \brief
   * Implementation of AbstractFunction::operator().
   */
  const double& operator()(const WorldVector<double>& x) const {
    static double result = 0.0;
    int dow = x.getSize();
    double r2 = (x*x);
    double ux = exp(-10.0*r2);
    result = -(400.0*r2 - 20.0*dow)*ux;
    return result;
  };
};

// // ===== main program // 
int main(int argc, char* argv[])
{
  FUNCNAME("main");

  // ===== check for init file =====
  TEST_EXIT(argc == 2)("usage: neumann initfile\n");

  // ===== init parameters =====
  Parameters::init(true, argv[1]);

  // ===== create and init the scalar problem ===== 
  ProblemScal neumann("neumann");
  neumann.initialize(INIT_ALL);

  // === create adapt info ===
  AdaptInfo *adaptInfo = NEW AdaptInfo("neumann->adapt", 1);

  // === create adapt ===
  AdaptStationary *adapt = NEW AdaptStationary("neumann->adapt",
					       &neumann,
					       adaptInfo);
  
  // ===== add boundary conditions =====
  neumann.addNeumannBC(1, NEW N);
  neumann.addDirichletBC(2, NEW G);

  // ===== create matrix operator =====
  Operator matrixOperator(Operator::MATRIX_OPERATOR, neumann.getFESpace());
  matrixOperator.addSecondOrderTerm(NEW Laplace_SOT);
  neumann.addMatrixOperator(&matrixOperator);

  // ===== create rhs operator =====
  int degree = neumann.getFESpace()->getBasisFcts()->getDegree();
  Operator rhsOperator(Operator::VECTOR_OPERATOR, neumann.getFESpace());
  rhsOperator.addZeroOrderTerm(NEW CoordsAtQP_ZOT(NEW F(degree)));
  neumann.addVectorOperator(&rhsOperator);

  // ===== start adaption loop =====
  adapt->adapt();

  neumann.writeFiles(adaptInfo, true);
}


